/*
** Jo Sega Saturn Engine
** Copyright (c) 2012-2017, Johannes Fetz (johannesfetz@gmail.com)
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Johannes Fetz nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL Johannes Fetz BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/***************************************************************************************************\
** Special Note : The 3D part on Jo Engine is still in development. So, some glitch may occur ;)
**                Btw, texture mapping for triangle base mesh (not quads) is experimental.
\***************************************************************************************************/

#include <jo/jo.h>
#include "ZT/ZT_COMMON.h"


/**Functions/variables added by XL2 **/
/**NOTE : Real-time gouraud shading - taken from Rockin'B demo**/

GOURAUDTBL	grtbl_workArea[500];
GOURAUDTBL		gourRealMax[GOUR_REAL_MAX];
Uint8			vwork[GOUR_REAL_MAX];
Uint8 enableRTG = 1;

/**Added by XL2 to use my own CD loading functions**/
#define     OPEN_MAX    (Sint32)5
#define     DIR_MAX     (Sint32)25
#define     RD_UNIT     (10)
#define     SECT_SIZE   (2048)
GfsDirTbl gfsDirTbl;
GfsDirName gfsDirName[DIR_MAX];
Uint32 gfsLibWork[GFS_WORK_SIZE(OPEN_MAX)/sizeof(Uint32)];
Sint32 gfsDirN;
void    ztCDinit(void)
{
    GFS_DIRTBL_TYPE(&gfsDirTbl) = GFS_DIR_NAME;
    GFS_DIRTBL_DIRNAME(&gfsDirTbl) = gfsDirName;
    GFS_DIRTBL_NDIR(&gfsDirTbl) = DIR_MAX;
    gfsDirN = GFS_Init(OPEN_MAX, gfsLibWork, &gfsDirTbl);
}
extern Sint8 SynchConst;
/****/

/**Taken from RB demo**/
static FIXED light[XYZ];
static ANGLE light_ang[XYZ];
void computeLight()
{
    FIXED light_init[XYZ] = { toFIXED(0.57735026), toFIXED(0.57735026), toFIXED(0.57735026) };

    slPushUnitMatrix();
    {
        slRotX(light_ang[X]);
        slRotY(light_ang[Y]);
        slRotZ(light_ang[Z]);
        slCalcVector(light_init, light);
    }
    slPopMatrix();
    light_ang[X] += DEGtoANG(0.5);
    light_ang[Y] += DEGtoANG(3.5);
    light_ang[Z] += DEGtoANG(0.5);
}

/**Added by XL2 - Very basic function to display a 3D model, position and radius are just for culling **/
void display_model(entity_t * model, bool UseRealtimeGouraud)
{
    /*
    //You could add parameters to the function and check if the model is onscreen for culling
    FIXED dist = slCheckOnScreen(model->pos, model->length[X]);
    if (dist < 0) return;*/
    int i;
    for (i=0; i<model->nbMeshes; i++)
    {
       if (UseRealtimeGouraud) slPutPolygonX((XPDATA*)model->pol[i], light);
       else slPutPolygon((PDATA*)model->pol[i]);
    }
}
/**End of added functions and global variables**/



Sint32 framerate;
ANGLE yy;

#include "SONICR.h"
void			    my_draw(void)
{
    yy+=(DEGtoANG(1.0));
    slPushMatrix();
    {
        slTranslate(0, 0, toFIXED(60.0));
        slRotY(yy);
        computeLight();

        display_model(&entities[0], 1);
        display_model(&entities[1], 0);

/*    If you want to try the generated header file instead, try with this
	  slPutPolygon((PDATA*)xpdata_0);
	  slPutPolygon((PDATA*)xpdata_1);
	  slPutPolygon((PDATA*)xpdata_2);
	  slPutPolygon((PDATA*)xpdata_3);
	  slPutPolygon((PDATA*)xpdata_4);
	  slPutPolygon((PDATA*)xpdata_5);
	  slPutPolygon((PDATA*)xpdata_6);
	  slPutPolygon((PDATA*)xpdata_7);
	  slPutPolygon((PDATA*)xpdata_8);
	  slPutPolygon((PDATA*)xpdata_9);*/

    }
    slPopMatrix();


    slPrintFX(toFIXED(jo_3d_get_polygon_count()), slLocate(0,10));
    slPrintFX(toFIXED(jo_3d_get_vertices_count()), slLocate(0,11));
    slPrintFX(toFIXED(jo_3d_get_displayed_polygon_count()), slLocate(0,12));
    //slPrintFX(jo_get_last_sprite_id()<<16, slLocate(0,13));
    return;
}

void my_vblank()
{
    if(enableRTG == 1)
        slGouraudTblCopy();
}

void			my_gamepad(void)
{
    if (jo_is_pad1_key_pressed(PER_DGT_ST))
        {
         if (enableRTG == 1) enableRTG=0;
         else enableRTG=1;
        }
}

void gameLoop(void)
{
   	while (1)
    {
        slUnitMatrix(0);
        my_gamepad();
        my_draw();
        slSynch();
    }
}

void			jo_main(void)
{
	jo_core_init(JO_COLOR_Black);

	/**Added by XL2 : turns off the TV while we load data**/
    slTVOff();
    /****/

    /**Added by XL2 **/
	slDynamicFrame(ON); //Dynamic framerate, when the VDP1 can't fully draw to the framebuffer in the allocated amount of time (1/60, 2/60, etc.) it will continue drawing when it's ON. Else it will try to complete drawing by skipping lines and details and finish in the allocated time. If the app runs well with the value at OFF, leave it at OFF!
    SynchConst=(Sint8)2;  //Framerate control. 1/60 = 60 FPS, 2/60 = 30 FPS, etc.
	framerate=2;
    /****/



	/**XL2**/
	ztCDinit(); /**Reinit the CD functions using my own. Might break Jo Engine loading functions*/
    ztFont2NBG3();  /**Swaps the NBG0 to NBG3 (NBG0 is much more flexible, so we use NBG3 for text only)**/
    memset_w((Sint16*)(VDP2_VRAM_A0), 0x0000, 262144);
    ztLoadVDP2bmp("NBG0.ZTI", (Uint16*)(VDP2_VRAM_A1), 1, bmNBG0);  /** Just to show you how to use the NBG0 with my custom function **/
    fadeIn();

    /**Leave some time to show the splash screen**/
    int i=0;
    while (i<60*4){
        i+=2; slSynch();
    }

    fadeOut(1); /**Again just to show you, it clears the BG when you put (1), else it keeps it (0) **/
    ztLoadVDP2bmp("BEACH.ZTI", (Uint16*)(VDP2_VRAM_A0), 3, bmNBG1); /**Loads the Beach to NBG1**/
    slScrPosNbg1(toFIXED(0.0), toFIXED(50.0)); //Offset the background here

    /**ZTP stands for Z-Treme polygonal model**/
    void * currentAddress = (void*)LWRAM;

    currentAddress = ztLoad3Dmodel((Sint8*)"SONIC.ZTP", currentAddress, &entities[0], 1);
    currentAddress = ztLoad3Dmodel((Sint8*)"TITLE.ZTP", currentAddress, &entities[1], 0);

    slInitGouraud(gourRealMax, GOUR_REAL_MAX, GRaddr, vwork);
	slIntFunction(my_vblank);
	/**Set your color here if you need one (depending on your light source)**/
    //slSetGouraudColor(CD_SemiRed);

    /**To set the Jo Engine callback (might affect gouraud shading calculations)**/
    /*jo_core_add_callback(my_gamepad);
	jo_core_add_callback(my_draw);*/

    slScrAutoDisp(NBG0ON | NBG1ON | NBG3ON | SPRON);

	fadeIn();/**Smooth fade in**/

	//jo_core_run();  //Uncomment for Jo Engine callback functions
	gameLoop();


}

/*
** END OF FILE
*/
